@@ -32,22 +32,17 @@
 #ifndef _SSL_FUNCTION_LIST_H_
 #define _SSL_FUNCTION_LIST_H_
 
-#if defined(VMCRYPTO_FIPS_COMPLIANT) || defined(OPENSSL_098)
-#define SHA1_T1 size_t
-#else
+#if !defined(VMCRYPTO_FIPS_COMPLIANT) && !defined(OPENSSL_097)
 #define SHA1_T1 unsigned long
+#else
+#define SHA1_T1 size_t
 #endif
 
-#ifdef OPENSSL_098
-#define SSL_U_C_A_T1 const unsigned char *
-#define SSL_C_U_C_A_T1 const unsigned char *
-#define EVP_VF_T1 const unsigned char *
-#define EVP_DU_T1 size_t
-#define PEM_W_B_RPK_T1 const RSA *
-#define D2I_PK_T1 const unsigned char **
-#define D2I_X509_T1 const unsigned char **
-#define HMAC_T1 size_t
-#else
+#ifdef OPENSSL_100
+#define STACK _STACK
+#endif
+
+#ifdef OPENSSL_097
 #define SSL_U_C_A_T1 unsigned char *
 #define SSL_C_U_C_A_T1 unsigned char *
 #define EVP_VF_T1 unsigned char *
@@ -56,6 +51,15 @@
 #define D2I_PK_T1 unsigned char **
 #define D2I_X509_T1 unsigned char **
 #define HMAC_T1 int
+#else
+#define SSL_U_C_A_T1 const unsigned char *
+#define SSL_C_U_C_A_T1 const unsigned char *
+#define EVP_VF_T1 const unsigned char *
+#define EVP_DU_T1 size_t
+#define PEM_W_B_RPK_T1 const RSA *
+#define D2I_PK_T1 const unsigned char **
+#define D2I_X509_T1 const unsigned char **
+#define HMAC_T1 size_t
 #endif
 
 #define VMW_SSL_VOID_FUNCTIONS_COMMON \
@@ -117,7 +121,8 @@
    VMW_SSL_FUNC(crypto, void, OPENSSL_add_all_algorithms_noconf, (void), ()) \
    VMW_SSL_FUNC(crypto, void, ERR_load_crypto_strings, (void), ()) \
    VMW_SSL_FUNC(crypto, void, EVP_CIPHER_CTX_init, (EVP_CIPHER_CTX *a), (a)) \
-   VMW_SSL_FUNC(crypto, void, EVP_MD_CTX_init, (EVP_MD_CTX *ctx), (ctx))
+   VMW_SSL_FUNC(crypto, void, EVP_MD_CTX_init, (EVP_MD_CTX *ctx), (ctx)) \
+   VMW_SSL_FUNC(crypto, void, ASN1_OBJECT_free, (ASN1_OBJECT *a), (a))
 
 #define VMW_SSL_RET_FUNCTIONS_COMMON \
    VMW_SSL_FUNC(crypto, int, CRYPTO_num_locks, (void), ()) \
@@ -433,6 +438,7 @@
    VMW_SSL_FUNC(crypto, RSA *, RSA_new, (void), ()) \
    VMW_SSL_FUNC(crypto, RSA *, EVP_PKEY_get1_RSA, (EVP_PKEY *pkey), (pkey)) \
    VMW_SSL_FUNC(crypto, X509 *, X509_new, (void), ()) \
+   VMW_SSL_FUNC(crypto, X509 *, X509_dup, (X509 *x), (x)) \
    VMW_SSL_FUNC(crypto, EVP_PKEY *, X509_get_pubkey, (X509 *x), (x)) \
    VMW_SSL_FUNC(crypto, int, EVP_MD_CTX_cleanup, (EVP_MD_CTX *ctx), (ctx)) \
    VMW_SSL_FUNC(crypto, int, EVP_DigestUpdate, (EVP_MD_CTX *ctx, const void *d, EVP_DU_T1 cnt), (ctx, d, cnt)) \
@@ -447,7 +453,11 @@
                                           (data, type, md, len)) \
    VMW_SSL_FUNC(crypto, int, X509_NAME_cmp, \
                 (const X509_NAME *a, const X509_NAME *b), (a, b)) \
-   VMW_SSL_FUNC(crypto, int, ASN1_TIME_print, (BIO *fp, ASN1_TIME *a), (fp, a))
+   VMW_SSL_FUNC(crypto, int, ASN1_TIME_print, (BIO *fp, ASN1_TIME *a), (fp, a)) \
+   VMW_SSL_FUNC(crypto, int, X509_get_ext_by_NID, \
+                (X509 *x, int nid, int lastpos), (x, nid, lastpos)) \
+   VMW_SSL_FUNC(crypto, X509_EXTENSION *, X509_get_ext, (X509 *x, int loc), (x, loc)) \
+   VMW_SSL_FUNC(crypto, void *, X509V3_EXT_d2i, (X509_EXTENSION *ext), (ext))
 
 #ifdef OPENSSL_098
 
